from qt import *
from Queue import Queue

from bbs import Topic
from renderer import Renderer
from kittywidget import *
from utils import *
import config
import os

class MessageWidget(KittyTabWidget):
    def __init__(self, parent):
        KittyTabWidget.__init__(self, parent)
        self._font = QFont(*config.get("gui-message-font", ()))
        self._newtab = True
        self._pages = {}
        self._queue = Queue()
        # Connect
        self.connect(self,
                     PYSIGNAL("tabRightPressed"),
                     self._tab_menu.popup)
        self.connect(self,
                     PYSIGNAL("tabMidPressed"),
                     self._close_pointed_page)
        self.connect(self,
                     PYSIGNAL("tabLeftDoubleClicked"),
                     self._reload_current_page)
        self.connect(self._tab_menu, PYSIGNAL("requestClose"), self._close)
        self.connect(self._tab_menu,
                     PYSIGNAL("requestReload"),
                     self._reload_pointed_page)
        # Start thread
        self._timer = QTimer()
        self._timer.start(100, False)
        self.connect(self._timer, SIGNAL("timeout()"), self._reduce_queue)

    def display(self, topic, messages, newtab=None):
        """Interface to display messages in topic
        
        This method actually submit messages into the local queue.  If 
        newtab is True, the new tab will be created for this topic.
        """
        assert isinstance(topic, Topic)
        if newtab==None:
            newtab = self._newtab
        self._queue.put((topic, messages, newtab))

    def set_font(self, font):
        self._font = font
        for view in self._pages.values():
            view.setFont(self._font)
        config.set("gui-message-font",
                   (unicode(font.family()), font.pointSize()))

    def font(self):
        return self._font

    def _reduce_queue(self):
        if not self._queue.empty():
            self._display(*self._queue.get())

    def _display(self, topic, messages, newtab):
        """Display the messages in topic
        
        If this window is already displaying the topic, the tab is
        raised.  Otherwise, create a new tab and display the list in
        the tab.
        """
        # If topic is being shown, use existing tab. Otherwise, create
        # new tab.
        if self._pages.has_key(topic):
            view = self._pages[topic]
        elif newtab==False and len(self._pages)>0:
            view = self.currentPage()
            self.changeTab(view, topic.title())
            self._forget_page(self.currentPage())
        else:
            view = QTextBrowser(self)
            view.setFont(self._font)
            self.connect(view,
                         SIGNAL("highlighted(const QString &)"),
                         self._hover)
            self.addTab(view, topic.title())

        self._pages[topic] = view
        # Draw
        view.setText(messages)
        self.showPage(view)

    def current_topic(self):
        widget = self.currentPage()
        d = dict([(w,t) for (t,w) in self._pages.items()])
        return d.get(widget, None)

    def _close_pointed_page(self, page, event):
        """Close page pointed by mouse cursor"""
        self._close(page)
    
    def _close(self, page_widget):
        """Close tab that contains page_widget

        In addition to call removePage() method, this method removes the 
        widget from self._pages dictionary.
        """
        self.removePage(page_widget)
        self._forget_page(page_widget)
        page_widget.close()

    def _reload_pointed_page(self, page):
        self._reload(dict([(w,t) for t,w in self._pages.items()]).get(page, None))

    def _reload_current_page(self, page):
        self._reload(self._current_topic())

    def _reload(self, topic):
        if topic!=None:
            self.emit(PYSIGNAL("requestSyncMessages"), (topic,))

    def _current_topic(self):
        widget = self.currentPage()
        d = dict([(w,t) for (t,w) in self._pages.items()])
        return d.get(widget, None)

    def _forget_page(self, widget):
        """Remove widget from self._pages"""
        topics = dict([(w,t) for t,w in self._pages.items()])
        del self._pages[topics.get(widget, None)]

    def _hover(self, href):
        if re.match("#\d+$", str(href)):
            index = int(str(href)[1:])
            topic = self.current_topic()
            html = "<qt type=detail encoding=utf8>%s</qt>" %\
              Renderer().render(topic, index, index+1)
            return

